const fs      = require('fs');
const gulp    = require('gulp');
const uglify  = require('gulp-uglify');
const concat  = require('gulp-concat');
const edit    = require('gulp-edit');
const debug   = require('gulp-debug');
const replace = require('gulp-replace');
const rename  = require('gulp-rename');
const bump    = require('gulp-bump');
const strnow  = require('strnow');
const merge   = require('merge-stream');

function get_conf()
{
	const getPackageJson = function () { return JSON.parse(fs.readFileSync('./bower.json', 'utf8')); };
	const pkg = getPackageJson();
	const result =  
	{
		'version' : pkg.version,
		'name'    : 'cbasejsclass'
	};
	return result;
}

function task_inc_patch()
{
	return merge
	(
		gulp.src('./bower.json')
			.pipe(bump())
			.pipe(gulp.dest('./'))
		,
		gulp.src('./package.json')
			.pipe(bump())
			.pipe(gulp.dest('./'))
	);
}

function task_build()
{
	const conf = get_conf();
	return gulp
		.src
		(
			[
				'bower_components/js-cookie/src/js.cookie.js',
				'bower_components/url.min/index.js',
				'src/console_ex.js',
				'bower_components/is_js/is.js',
				'src/jquery-2.2.4.min.js',
				'src/form_utils.js',
				'src/php.js',
				'src/inc_utils.js',
				'src/dev_utils.js',
				'src/jquery_cjs_middle.js',
				'src/jquery_cjs_before.js',
				'src/ui.js',
				'src/jquery.easing.js',
				'src/cjsbaseclass.js'
			]
		)
		.pipe(concat('cjsbaseclass.js'))
		.pipe(replace("self.version = '1.0.0';", "self.version = '" + conf.version + "';"))
		.pipe(edit(function(src, cb) {src = '/* (https://bitbucket.org/marceloxp/cjsbaseclass) Version: ' + conf.version + ' Last modified: ' + strnow.get() + ' */ \n/* exported CjsBaseClass */ \n' + src; cb(null, src);}))
		.pipe(gulp.dest('dist/'))
		.pipe(uglify())
		.pipe(edit(function(src, cb) {src = '/* (https://bitbucket.org/marceloxp/cjsbaseclass) Version: ' + conf.version + ' Last modified: ' + strnow.get() + ' */ \n/* exported CjsBaseClass */ \n' + src; cb(null, src);}))
		.pipe(rename({ suffix: '.min'}))
		.pipe(gulp.dest('dist/'))
		.on('end', function() { console.log('Gulp Task', 'Build task complete'); })
	;
}

function task_build_slim()
{
	const conf = get_conf();
	return gulp
		.src
		(
			[
				'bower_components/js-cookie/src/js.cookie.js',
				'bower_components/url.min/index.js',
				'src/console_ex.js',
				'bower_components/is_js/is.js',
				'src/form_utils.js',
				'src/php.js',
				'src/inc_utils.js',
				'src/dev_utils.js',
				'src/jquery_cjs_middle.js',
				'src/jquery_cjs_before.js',
				'src/ui.js',
				'src/jquery.easing.js',
				'src/cjsbaseclass.js'
			]
		)
		.pipe(concat('cjsbaseclass.slim.js'))
		.pipe(replace("self.version = '1.0.0';", "self.version = '" + conf.version + "';"))
		.pipe(edit(function(src, cb) {src = '/* (https://bitbucket.org/marceloxp/cjsbaseclass) Version: ' + conf.version + ' (slim) Last modified: ' + strnow.get() + ' */ \n/* exported CjsBaseClass */ \n' + src; cb(null, src);}))
		.pipe(gulp.dest('dist/'))
		.pipe(uglify())
		.pipe(edit(function(src, cb) {src = '/* (https://bitbucket.org/marceloxp/cjsbaseclass) Version: ' + conf.version + ' (slim) Last modified: ' + strnow.get() + ' */ \n/* exported CjsBaseClass */ \n' + src; cb(null, src);}))
		.pipe(rename({ suffix: '.min'}))
		.pipe(gulp.dest('dist/'))
		.on('end', function() { console.log('<-- Gulp Task', 'Dist Slim task complete'); })
	;
}

function task_demo()
{
	return gulp
		.src('src/main.js')
		.pipe(replace('${1:namespace}', 'MyNameSpace'))
		.pipe(replace('${2:object_name}', 'myclass'))
		.pipe(gulp.dest('demo/cjsbaseclass/'))
		.pipe(gulp.dest('dist/'))
		.on('end', function() { console.log('Gulp Task', 'Sublime Main Demo task complete'); })
	;
}

function task_snippet_1x()
{
	return gulp
		.src('src/main-1x.js')
		.pipe
		(
			edit
			(
				function(src, cb)
				{
					src =
						'<snippet>\n\t<tabTrigger>class_instanced_main</tabTrigger>\n\t<scope>source.js</scope>\n\t<description>1x Base CjsBaseClass</description>\n\t<content>\n\t<![CDATA[\n' +
						src +
						']]>\n</content>\n</snippet>'
					;
					cb(null, src);
				}
			)
		)
		.pipe(replace('$', '\\$'))
		.pipe(replace('\\${', '${'))
		.pipe(rename('base_js_main_class-1x.sublime-snippet'))
		.pipe(gulp.dest('dist/'))
		.on('end', function() { console.log('Gulp Task', 'Sublime Main Snippet task complete (version 1x)'); })
	;
}

function task_snippet_2x()
{
	return gulp
		.src('src/main.js')
		.pipe
		(
			edit
			(
				function(src, cb)
				{
					src =
						'<snippet>\n\t<tabTrigger>class_instanced_main</tabTrigger>\n\t<scope>source.js</scope>\n\t<description>2x Base CjsBaseClass</description>\n\t<content>\n\t<![CDATA[\n' +
						src +
						']]>\n</content>\n</snippet>'
					;
					cb(null, src);
				}
			)
		)
		.pipe(replace('$', '\\$'))
		.pipe(replace('\\${', '${'))
		.pipe(rename('base_js_main_class.sublime-snippet'))
		.pipe(gulp.dest('dist/'))
		.on('end', function() { console.log('Gulp Task', 'Sublime Main Snippet task complete (version 2x)'); })
	;
}

async function task_npm_publish()
{
	const conf = get_conf();
	const child_process = require('child_process');
	let result, command;
	const commands = 
	[
		'git add *',
		'git commit -m "Upgrade version"',
		'git push origin master',
		'npm publish',
		'git tag v' + conf.version,
		'git push origin --tags'
	];

	for(let k = 0; k < commands.length; k++)
	{
		command = commands[k];
		console.log('--> Executing command', command);
		try
		{
			result = child_process.execSync(command);
			console.log(result);
		}
		catch(err)
		{
			console.log(err.message);
		}
		console.log('<-- Done Executing command', command);
	}
	return true;
}

async function task_git_connect()
{
	const command = 'ssh-add ~/.ssh/console';
	const result = require('child_process').execSync(command).toString();
	console.log(result);
	return true;
}

const group_compile_sub   = gulp.series(task_demo, task_snippet_1x, task_snippet_2x);
const group_compile_src   = gulp.series(task_build, task_build_slim, group_compile_sub);
const group_publish       = gulp.series(task_git_connect, task_inc_patch, task_build, task_build_slim, group_compile_sub, task_npm_publish);

exports.task_inc_patch    = task_inc_patch;
exports.task_build        = task_build;
exports.task_build_slim   = task_build_slim;
exports.task_demo         = task_demo;
exports.task_snippet_1x   = task_snippet_1x;
exports.task_snippet_2x   = task_snippet_2x;
exports.task_git_connect  = task_git_connect;
exports.group_compile_sub = group_compile_sub;
exports.group_compile_src = group_compile_src;
exports.group_publish     = group_publish;